const crypto = require('crypto');
const axios  = require('axios');

const hash256 = str => {
    return crypto
        .createHash('sha256')
        .update(str, 'utf8')
        .digest('hex');
};

const sign = (key, msg, digest = 'buffer') => {
    return crypto
        .createHmac('sha256', key)
        .update(msg, 'utf8')
        .digest(digest);
};

const getSignatureKey = (key, dateStamp, regionName, serviceName) => {
    const $kDate    = sign(`AWS4${key}`, dateStamp);
    const $kRegion  = sign($kDate, regionName);
    const $kService = sign($kRegion, serviceName);
    const $kSigning = sign($kService, 'aws4_request');
    return $kSigning;
};

const apiRequest = ({
    ak,
    sk,
    action  = 'ClassifyImageGuard',
    method  = 'POST',
    service = 'kir',
    host    = 'kir.api.ksyun.com',
    version = '2019-01-18',
    post_data
}) => {
    const region             = 'cn-beijing-6';
    const endpoint           = `http://${host}`;
    const contentType        = 'application/json';
    const request_parameters = `Action=${action}&Version=${version}`;

    const canonical_uri         = '/';
    const canonical_querystring = request_parameters;
    let amzdate                 = `${new Date().toISOString().split('.')[0]}Z`;
    amzdate                     = amzdate.replace(/\-|:/g, '');
    const datestamp             = amzdate.split('T')[0];
    const algorithm             = 'AWS4-HMAC-SHA256';
    const credential_scope      = `${datestamp}/${region}/${service}/aws4_request`;
    const signed_headers = 'content-type;host;x-amz-date';

    const canonical_headers = `content-type:${contentType}\nhost:${host}\nx-amz-date:${amzdate}\n`;

    const payload_hash      = hash256(JSON.stringify(post_data));
    const canonical_request = `${method}\n${canonical_uri}\n${canonical_querystring}\n${canonical_headers}\n${signed_headers}\n${payload_hash}`;
    const string_to_sign = `${algorithm}\n${amzdate}\n${credential_scope}\n${hash256(canonical_request)}`;

    const signing_key          = getSignatureKey(sk, datestamp, region, service);
    const signature            = sign(signing_key, string_to_sign, 'hex');
    const authorization_header = `${algorithm} Credential=${ak}/${credential_scope}, SignedHeaders=${signed_headers}, Signature=${signature}`;
    const headers = {
        'content-type' : contentType,
        Authorization : authorization_header,
        'x-amz-date' : amzdate
    };

    return axios.post(`${endpoint}?${canonical_querystring}`, post_data, { headers });
};

const ak        = 'ak from www.ksyun.com';
const sk        = 'sk from www.ksyun.com';
const post_data = {
    guard_id : '1547778774476511751',
    image_url : 'https://ks3-cn-beijing.ksyun.com/datasets/sdk/chenshuibian.png'
};

apiRequest({ ak, sk, post_data })
    .then(res => {
        console.log('请求结果：', res.data);
    })
    .catch(err => {
        console.log('错误信息:', err.response.data);
    });
